import {Component} from "./Component.class";
import {Constants} from "./Constants";

/**
* @class
* @constructor
*/
export class ComponentTransform extends Component {
    constructor() {
        super();
	
        this.type = Constants.COMPONENT_TYPES.TRANSFORM;
        this.node = null;
        this.gl = null;


        this.mModelMatrix_Position = $M16([1.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,1.0]);
        this.mModelMatrix_Rotation = $M16([1.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,1.0]);
    }
	
	/**
	 * initialize
	 * @param {Node} nod
	 * @param {WebGLRenderingContext} glCtx.
	 * @override
	 */
	initialize(nod, glCtx) {
        this.node = nod;
        this.gl = glCtx;
	};	
	
	/**
	 * getMatrixPosition
	 * @returns {StormM16}
	 */
	getMatrixPosition() {
		return this.mModelMatrix_Position;
	};

    /**
     * getPosition
     * @returns {StormV3}
     */
    getPosition() {
        return $V3([this.mModelMatrix_Position.e[3], this.mModelMatrix_Position.e[7], this.mModelMatrix_Position.e[11]]);
    };

    /**
     * getXPosition
     * @returns {number}
     */
    getPosX() {
        return this.mModelMatrix_Position.e[3];
    };

    /**
     * getYPosition
     * @returns {number}
     */
    getPosY() {
        return this.mModelMatrix_Position.e[7];
    };

    /**
     * getZPosition
     * @returns {number}
     */
    getPosZ() {
        return this.mModelMatrix_Position.e[11];
    };

    /**
     * setPosition
     * @param {StormV3} pos
     */
    setPosition(pos) {
        this.mModelMatrix_Position.e[3] = pos.e[0];
        this.mModelMatrix_Position.e[7] = pos.e[1];
        this.mModelMatrix_Position.e[11] = pos.e[2];
    };

    /**
     * setXPosition
     * @param {number} val
     */
    setPosX(val) {
        this.mModelMatrix_Position.e[3] = val;
    };

    /**
     * setXPosition
     * @param {number} val
     */
    setPosY(val) {
        this.mModelMatrix_Position.e[7] = val;
    };

    /**
     * setXPosition
     * @param {number} val
     */
    setPosZ(val) {
        this.mModelMatrix_Position.e[11] = val;
    };

	/**
	 * getMatrixRotation
	 * @returns {StormM16}
	 */
	getMatrixRotation() {
		return this.mModelMatrix_Rotation;
	};
}
global.ComponentTransform = ComponentTransform;
module.exports.ComponentTransform = ComponentTransform;